#!/usr/bin/python
# -*- coding:utf-8 -*-

import sys
from logging import *

TYPE_UNKNOWN = 0
TYPE_SETTLEMENT = 1
TYPE_ZONE = 2
TYPE_PROVINCE = 3
TYPE_GLOB = 4

class patt():
    def __init__(self, name, replacement):
        self.name = name;
        self.replacement = replacement;

pref_settlement = [
    patt("д.", None),
    patt("Д.", None),
    patt("дер", None),
    patt("дер.", None),
    patt("с.", None),
    patt("г.", None),
    patt("п.", None),
    patt("ст.", None),
    patt("Ст.", None),
    patt("СТ.", None),
    patt("с.", None),
    patt("пгт.", None),
    patt("пгт", None),
    patt("пос", None),
    patt("пос.", None),
    patt("х.", None),
    patt("м.", None),
    patt("з/с", None),
    patt("з/с.", None),
    patt("с/з", None),
    patt("с/з.", None),
    patt("с-з", None),
    patt("с-з.", None),
    patt("с/с", None),
    patt("с/с.", None),
    patt("с/х", None),
    patt("с/х.", None),
    patt("л/п", None),
    patt("л/п.", None),
    patt("л/у", None),
    patt("л/у.", None),
    patt("л/уч", None),
    patt("л/уч.", None),
    patt("уч.", None),
    patt("ф.", None),
];

pref_zone = [
    patt("р-н", "район"),
    patt("р-он", "район"),
    patt("район", "район"),
    patt("округ", "округ"),
];

pref_province = [
    patt("республика", "республика"),
    patt("республики", "республика"),
    patt("респ-ка", "республика"),
    patt("р.", "республика"),
    patt("р-ка", "республика"),
    patt("респ", "республика"),
    patt("респ.", "республика"),
    patt("Респ.", "республика"),
    patt("Респ", "республика"),
    patt("республика", "республика"),
    patt("обл.", "область"),
    patt("обл", "область"),
    patt("область", "область"),
    patt("республика", "республика"),
    patt("Республика", "республика"),
]

suf_settlement = [
    "л/п",
    "л/п,",
]

suf_zone = [
    patt("р-н", "район"),
    patt("р-он", "район"),
    patt("р-на", "района"), #KILL ME:(
    patt("район", "район"),
    patt("округ", "округ"),
];

suf_province = [
    patt("обл.", "область"),
    patt("обл", "область"),
    patt("область", "область"),
    patt("респ.", "республика"),
    patt("респ", "республика"),
    patt("Респ.", "республика"),
    patt("Респ", "республика"),
    patt("республика", "республика"),
    patt("Республика", "республика"),
    patt("край", "край"), #не уверен здесь ли надо
]

suf_glob = [
    patt("АССР", "АССР"),
    patt("ССР", "АССР"),
]

pref_types = [
    [pref_settlement, TYPE_SETTLEMENT],
    [pref_zone, TYPE_ZONE],
    [pref_province, TYPE_PROVINCE],
]
suf_types = [
    [suf_zone, TYPE_ZONE],
    [suf_province, TYPE_PROVINCE],
    [suf_glob, TYPE_GLOB]
]

def normalize(s):
    """ return normalized address started with root or print warn"""
    s = s.rstrip('\n');
    s = s.rstrip('\r');
    s = s.rstrip('.');
    s = s.replace(',', ' ').replace(' - ', '-').replace('.', '. ');
    s = s.replace('\t', ' ');
    return s;

def split_words(s):
    return s.split();

class entity:
    def __init__(self, name, type = TYPE_UNKNOWN):
        self.type = type;
        self.name = name

def search_replacement(s, pattern_lst):
    "return list [type, repl] if finds replacement in list of patt or [TYPE_UNKNOWN, None]"
    #print "debug: s = %s" % [s]
    #print pattern_lst
    ls = s.lower();
    for patterns in pattern_lst:
        for pi in patterns[0]:
            if ls == pi.name:
                return [patterns[1], pi.replacement];
    return [TYPE_UNKNOWN, None];

def get_next_entity(lst, i):
    """return number of processed items and entity elem [n, elem]"""
    #print 'debug:lst[i] = %s' %lst[i]
    if i + 1 < len(lst):
        t, repl = search_replacement(lst[i], pref_types);
        if t != TYPE_UNKNOWN:
            s = "";
            if repl != None:
                s = repl + " ";
            s += lst[i + 1]
            ent = entity(s, t)
            return [2, ent]
        t, repl = search_replacement(lst[i + 1], suf_types)
        if t != TYPE_UNKNOWN:
            s = lst[i];
            if repl != None:
                s= s + " " + repl
            ent = entity(s, t)
            return [2, ent]
    ent = entity(lst[i], TYPE_UNKNOWN)        
    return [1, ent]

def get_entity_list(s):
    s = normalize(s);
    #print '[+] debug, after normalize: \'%s\'' % s
    lst = split_words(s);
    #print '[+]debug split list of words: %s' % lst
    i = 0;
    res = []
    while i < len(lst):
        [n, v] = get_next_entity(lst, i)
        i += n
        res.append(v)
    return res

adj_endings = [
    "ое",
    "ая",
    "ие",
    "ий",
    "ый",
    "ые",
    "яя"
];

def is_adjective(s):
    if len(s) < 4:
        return False;
    for item in adj_endings:
        """ Так как в данном случае используется utf-8,
        русские символы кодируются 2 байтами.
        """
        #print "debug: %s" % s[-4:]
        if s[-4:] == item:
            #Probably yes
            return True;
    #probably no
    return False;

known_abbr_lst = [
    patt("РК", "республика Коми"),
    patt("РКоми", "республика Коми"),
]

def replace_known_abbr(lst):
    for item in lst:
        for abbr in known_abbr_lst:
            if item.name == abbr.name:
                item.name = abbr.replacement;
                #replace me later
                item.type = TYPE_GLOB

def post_process_lst(lst):
    replace_known_abbr(lst);
    if len(lst) < 2:
        return
    i = 0;
    while i < len(lst) - 1:
        if is_adjective(lst[i].name) and lst[i + 1].type == TYPE_UNKNOWN:
            log(DEBUG, "adjective finded name %s + %s",
                lst[i].name, lst[i + 1].name);
            lst[i].name = lst[i].name + " " + lst.pop(i + 1).name;
        i += 1
         
basicConfig(level = ERROR);

for line in sys.stdin.readlines():
    lst = get_entity_list(line)
    log(DEBUG, "debug: lst = %s", lst);
    post_process_lst(lst)
    for i in lst:
        sys.stdout.write("%s (%d) =>" % (i.name, i.type));
    sys.stdout.write("\n");

